# -*- coding: utf-8 -*-
"""
Created on Sat Apr 18 22:37:36 2020

@author: stefa
"""
from ._helper import is_float, search_char
from calc2tex import _formula

singleun  = {"m": "\\meter", "g": "\\gram", "A": "\\ampere", "C": "\\coulomb", "K": "\\kelvin", "s": "\\second", "J": "\\joule", "N": "\\newton",
             "V": "\\volt", "W": "\\watt", "h": "\\hour", "l": "\\litre", "t": "\\tonne", "d": "\\day", "a": "\\year", "°": "\\degree"}
singlepre = {"p": "\\pico", "n": "\\nano", "m": "\\milli", "c": "\\centi", "d": "\\dezi", "h": "\\hecto", "k": "\\kilo",
             "M": "\\mega", "G": "\\giga", "T": "\\tera", "P": "\\peta"}
doubleun  = {"ha": "\\hectare", "Pa": "\\pascal", "cd": "\\candela", "Bq": "\\bequerel", "Hz": "\\hertz",
             "lm": "\\lumen", "Sv": "\\sievert", "dB": "\\decibel", "°C": "\\degreeCelsius"}
doublepre = {"mu": "\\micro", "da": "\\deca"}
tribleun  = {"min": "\\minute", "mol": "\\mole", "bar": "\\bar", "ohm": "\\ohm", "rad": "\\radian",
             "gon": "\\gon"}

charUnits   = ["/", "*"]
charFormula = ["/", "*", "(", ")", "+", "-", "%"]


def read_file(filename: str) -> dict:
    """
    Parses a txt-file into a dictionary.

    Parameters
    ----------
    filename : str
        The path to a txt-file, containing a semicolon-separated list.

    Returns
    -------
    dict
        A dictionary, containing the parsed file.

    """
    data: dict = {"used_bib": []}                       #creates an empty container for the information in the file
    data: dict = {}
    input_list: list = []                               #a list for holding the pre-processed lines 
    
    with open(filename) as file:
        for line in file:                               #iterates over the lines in file
            save: list = line.split(";")                #splits line on every semi-colon
            save = [part.strip() for part in save]      #removes leading and trailing whitespace on every substring
            if save[0] == "":                           #empty lines and lines starting with the hash character are ignored
                continue
            elif save[0][0] != "#":
                input_list.append(save)
    
    #TODO in formel alle Leerzeichen entfernen
    #TODO Möglichkeit tex_var wegzulassen, wenn gleich
    #TODO bibs in eigene Liste, initialisierung anpassen, klasse.table anpassen
    #TODO Leerzeichen ersetzen nicht in Formel da dort if's etc.
    #TODO save input as dict with line-numbers, for exception handeling
    #TODO other decision making: look if line[2] can be converted to float
        #use: line[2].replace('.', '', 1).isdigit() no exponents
        #try catch- block: float(line[0]) as a helper function
    
    for line in input_list:                                 #extracts information from pre-processed file into data-container
        if len(line) == 1:                                  #differentiats different cases by length of list
            bibs = line[0].split(",")                       #splits at first on commas
            _, first = bibs.pop(0).split(":")               #the first element, which uses another delimiter, is seperatly processed
            bibs.insert(0, first)                           #and then added back again to the list
            bibs = [part.strip(" ") for part in bibs]       #removes whitespaces
            #data["used_bib"].extend(bibs)
            #TODO zweites dict mit Werten von Bibs, auf Reihenfolge von Einfügen achten falls Doppelkey,
                #erst im aktuellen Verzeichnis schauen-> Aufbau und Verarbeitung wie read-file, dann im Modulverzeichnis Biblio suchen
                #dort als json oder txt speichern, letzteres geringerer Platzbedarf, langsameres parsen; abhängig von Dateiendung verarbeiten
            #TODO keine Endung: Standardendung festlegen wahrscheinlich json -> so schon berechnetes txt-später verwendbar
            #TODO gewünschte Präzision angeben, verwendung dann in val_in??
            #TODO erste drei als reference by pos, Rest mit key = schreiben
        elif len(line) == 4:
            data[line[0]] = {"var": "val", "tex_var": line[1], "res": float(line[2]), "unit": line[3], "tex_un": None}
            #TODO line[3] only to next hash character
        elif len(line) == 5:
            data[line[0]] = {"var": "form", "tex_var": line[1], "res": None, "unit": line[4], "tex_un": None, "type": line[2], "form": line[3]}
    
    return data


#TODO unit conversion in eigenes Modul auslagern??
def subunit_to_tex(split: str) -> str:
    """
    Converts part of a unit from the symbol to the command of the siunitx-package

    Parameters
    ----------
    split : str
        Part of the inputed unit, one symbol or a number.

    Returns
    -------
    str
        The command from the siunitx-package representing the symbol.

    """
    length = len(split)
    if is_float(split):
        return "\\tothe{" + split + "}"
    
    if length == 1:
        return singleun[split]
    elif length == 2:
        try:
            return doubleun[split]
        except:
            pass
        return singlepre[split[0]] + singleun[split[1]]
    elif length == 3:
        try:
            return tribleun[split]
        except:
            pass
        try: 
            return doublepre[split[0:2]] + singleun[split[2]]
        except:
            pass
        return singlepre[split[0]] + doubleun[split[1:3]]
    elif length == 4:
        try:
            return singlepre[split[0]] + tribleun[split[1:4]]
        except:
            pass
        return doublepre[split[0:2]] + doubleun[split[2:4]]

    
    return None



def unit_to_tex(unit: str) -> str:
    """
    Converts the unit from the input-file to a string, that can be used by the siunitx-package.

    Parameters
    ----------
    unit : str
        The complete unit, as extracted from the file.

    Returns
    -------
    str
        A string, that can be inputed in LaTeX.

    """
    if unit == "-" or "":
        return "-"
    
    truth = True
    index_last = 0
    tex_unit = ""
    
    while truth:
        index_next = search_char(unit, index_last, charUnits)
        
        if index_next - index_last != 0:
            tex_unit += subunit_to_tex(unit[index_last:index_next])
        if index_next == len(unit):
            break
        if unit[index_next] == "/":
            tex_unit += "\\per"
        index_last = index_next + 1
            
    
    return tex_unit



def calculate(data: dict) -> dict:
    """
    Takes the dictionary with the inputs from the txt-file, and returns
    a dictionary with all values and LaTeX-strings calculated.

    Parameters
    ----------
    data : dict
        The unprocessed dictionary.

    Returns
    -------
    dict
        A dictionary, with every value calculated.

    """
    for key in data.keys():
        data[key]["tex_un"] = unit_to_tex(data[key]["unit"])
        if data[key]["var"] == "form":
            data[key]["res"], data[key]["var_in"], data[key]["val_in"] = _formula.main(data[key]["form"], data)
        
    return data


def main(filename: str) -> dict:
    """
    Reads and calculates a fully filled dictionary to use in the class.

    Parameters
    ----------
    filename : str
        The filename of a txt-file containing the inputs.

    Returns
    -------
    dict
        A dictionary with springs for displaying in LaTeX and values.

    """
    data = read_file(filename)
    return calculate(data)




# -*- coding: utf-8 -*-
"""
Created on Sun Apr 19 09:14:34 2020

@author: stefa
"""


#TODO Konstanten und anpassbare Sachen hier auslagern einheitendatenbank


language = {"DE": 
                {"table": 
                     {"header": "Eingangsgrößen", "var": "Variable", "val": "Wert", "unit": "Einheit"}
                },
            "EN": 
                {"table": 
                     {"header": "input values", "var": "variable", "val": "value", "unit": "unit"}
                }
            }

accuracy = 3

mult_sign = "\\cdot" #others: \times, \star, \ast


# -*- coding: utf-8 -*-
"""
Created on Thu May  7 22:36:15 2020

@author: stefa
"""
from ._helper import search_bracket, search_char
from .klasse import Calc2tex

def tex_to_tex(in_file: str, out_file: str) ->None:
    commands = []
    #TODO variable vor Pytex und istgleich in commands speichern
    with open(in_file) as stdin, open(out_file, "w") as stdout:
        for line in stdin:
            while True:
                if "Calc2tex(" in line:
                    index = line.index("Calc2tex(")
                    bracket = search_bracket(line, index+5, 1)
                    end = line.rindex("=", 0, index)
                    while line[end] == " ":
                        end -= 1
                
                    start = line.rfind(" ", 0, end-1) + 1
                    if start == 0:
                        start = line.rindex("\t", 0, end-1) + 1
                
                    commands.append(line[start:end-1] + ".")
                    exec(line[start:bracket+1])
                    line = line.replace(line[start:bracket+1], "")
                else:
                    break
               
            for command in commands:
                while True:
                    if command in line:
                        index = line.index(command)
                        opening = search_char(line, index, ["("])
                        closing = search_bracket(line, opening, 1)
                        line = line.replace(line[index:closing+1], eval(line[index:closing+1]))
                    else:
                        break
            
            stdout.write(line)
            
            


# -*- coding: utf-8 -*-
"""
Created on Sat May  2 21:06:53 2020

@author: stefa
"""


def is_float(number: str) -> bool:
    """Checks if the input-string can be converted ta a valid number."""
    try:
        float(number)
    except:
        return False
    else:
        return True
    
    
#TODO chars as tuple 
def search_char(string: str, pos: int, chars: list) -> int:
    """
    Searches for the index of the next char out of the chars-list.

    Parameters
    ----------
    string : str
        A formula.
    pos : int
        The index, at which searching should be started.
    chars : list
        A list containing strings.

    Returns
    -------
    int
        The index of the next char or if there is no char found the lenght of the string.

    """
    indices = []
    for char in chars:
        try:
            indices.append(string.index(char, pos))
        except:
            pass
    
    if indices == []:
        return len(string)
    else:
        return min(indices)



def search_bracket(string: str, pos: int, direction: int) -> int:
    """
    searches the corresponding opening or closing bracket

    Parameters
    ----------
    string : str
        A formula.
    pos : int
        The Index of the bracket, at which searching should be started.
    direction : int
        The direction at which should be searched.

    Returns
    -------
    int
        Index of the corresponding bracket.

    """
    numcorr, numdiff = 0, 0
    newpos = pos
    if direction == -1:
        while True:
            index = string.rfind("(", 0, newpos)
            # if index == -1:
            #     raise Exception
            numdiff += string.count(")", index, newpos)
            if numdiff == numcorr:
                return index
            numcorr += 1
            newpos = index
                
    elif direction == 1:
        newpos += 1
        while True:
            index = string.find(")", newpos)
            numdiff += string.count("(", newpos, index)
            if numdiff == numcorr:
                return index
            numcorr += 1
            newpos = index + 1







# -*- coding: utf-8 -*-
"""
Created on Sat Apr 18 23:08:31 2020

@author: stefa
"""
from ._helper import is_float, search_char, search_bracket
from .settings import mult_sign
#Imports for evaluating formula:
import numpy as np
from calc2tex import trigo


def split_mult(string: str) -> str:
    """
    Splits the string on the highest order factors und passes them again into the split_sum-function.

    Parameters
    ----------
    string : str
        A formula, which is preprocessed by split_sum and split_quo.

    Returns
    -------
    str
        The input with the single factors processed.

    """
    index_last = 0
    back = ""
    
    while True:
        index_start = search_char(string, index_last, ["("])
        if index_start == len(string):
            back += string[index_last:index_start]
            break
        
        index_next = search_bracket(string, index_start, 1)
        
        back += "".join((string[index_last:index_start+1], split_sum(string[index_start+1:index_next]), string[index_next]))
        
        index_last = index_next + 1
    
    return back



def split_quo(string: str) -> str:
    """
    Converts fractions.

    Parameters
    ----------
    string : str
        The formula preprocessed by split_funct, split_expo and split_sum.

    Returns
    -------
    str
        Processed formula.

    """
    #TODO ?/?/? berücksichtigen -> \frac{?}{?*?}
    index_last, index_start = 0, 0
    back = ""
    
    if string[0] == "+" or string[0] == "-":
        index_last = 1
        back += string[0]
        
    while True:
        while True:
            index_next = search_char(string, index_start, ["/"])
            index_start = index_next + 1
            if string.count("(", 0, index_next) - string.count(")", 0, index_next) == 0:
                break
            
        if index_next == len(string):
            back += split_mult(string[index_last:])
            break
    
        if string[index_next+1] == "/":
            intdiv = True
            index_start += 1
            start, end = index_next - 1, index_next + 2
        else:
            intdiv = False
            start, end = index_next - 1, index_next + 1
        
        if string[start] == ")":
            start = search_bracket(string, start, -1)
            numerator = split_sum(string[start+1:index_next-1])
        else:
            numerator = string[start]
        
        if string[end] == "(":
            end = search_bracket(string, end, 1)
            if intdiv:
                denominator = split_sum(string[index_next+3:end])
            else:
                denominator = split_sum(string[index_next+2:end])
        else:
            denominator = string[end]
        
        before = split_mult(string[index_last:start])
        if intdiv:
            back += "".join((before,"\\left\\lfloor\\frac{", numerator, "}{", denominator, "}\\right\\rfloor "))
        else:
            back += "".join((before,"\\frac{", numerator, "}{", denominator, "}"))
        index_last = end + 1
      
    return back



def split_sum(string: str) -> str:
    """
    Splits the formula on single summands and processes one a time.

    Parameters
    ----------
    string : str
        The formula preprocessed by split_funct and split_expo.

    Returns
    -------
    str
        Processed formula.

    """
    if len(string) == 0:
        return string
    #TODO muss index_last auf 1 gesetzt werden, immer auf 1 -> if bedingung in schleife weg
    if string[0] == "+" or string[0] == "-":
        index_last, index_start = 1, 1
    else:
        index_last, index_start = 0, 0
    back = ""
    
    while True:
        index_next = search_char(string, index_start, ["+", "-"])
        if string.count("(", 0, index_next) - string.count(")", 0, index_next) == 0:
            if index_last == 0:
                #print(string, string[index_last:index_next])
                back += split_quo(string[index_last:index_next])
            else:
                #print(string, string[index_last-1:index_next])
                back += split_quo(string[index_last-1:index_next])
            index_last = index_next + 1
        
        #print("exit")
        if index_next == len(string):
            break
        
        index_start = index_next + 1
       
    return back 



def split_expo(string: str) -> str:
    """
    Takes a string and processes all exponents.

    Parameters
    ----------
    string : str
        Formula preprocessed by split_funct.

    Returns
    -------
    str
        Processed formula.

    """
    index_last = 0
    expos = []
    
    while True:
        index_next = search_char(string, index_last, ["**"])
        if index_next == len(string):
            break
        
        start, end = index_next - 1, index_next + 2
        
        if string[start] == ")":
            start = search_bracket(string, start, -1)
            base = "(" + split_sum(string[start+1:index_next-1]) + ")"
        else:
            base = string[start]
            
        if string[end] == "(":
            end = search_bracket(string, end, 1)
            if "**" in string[index_next+3:end]:
                power = split_expo(string[index_next+3:end])
            else:
                power = split_sum(string[index_next+3:end])
        else:
            power = string[end]
        
        expo_in_base = string.count("§", start, index_next)
        
        expos.insert(len(expos)- expo_in_base, "".join((base, "^{", power, "}")))
        string = string.replace(string[start:end+1], "§", 1)
        
        index_last = start
    
    back = split_sum(string)
    
    for expo in expos:
        back = back.replace("§", expo, 1)
        
    return back



def split_funct(string: str) -> str:
    """
    Searches for functions inside string and separatly converts them.

    Parameters
    ----------
    string : str
        The unprocessed formula.

    Returns
    -------
    str
        Nearly to LaTeX converted formula.

    """
    functions = []
    index_last = 0
    
    while True:
        index_start = search_char(string, index_last, ["?(", "~("])
        
        if index_start >= len(string):
            break
        
        index_next = search_bracket(string, index_start+1, 1) #von index_start+2 geändert
        
        if "?(" in string[index_start+2:index_next+1] or "~(" in string[index_start+2:index_next+1]:
            if string[index_start] == "~":
                functions.append("?{" + split_funct(string[index_start+2:index_next]) + "}")
            else:
                
                functions.append("?(" + split_funct(string[index_start+2:index_next]) + ")")
        else:
            if string[index_start] == "~":
                functions.append("?{" + split_expo(string[index_start+2:index_next]) + "}")
            else:
                functions.append(split_expo(string[index_start:index_next+1]))
                
        string = string.replace(string[index_start:index_next+1], "!", 1)
        index_last = index_start
      
    back = split_expo(string)
    
    for funct in functions:
        back = back.replace("!", funct, 1)
    
    return back



def find_vars(formula: str) -> (str, list):
    """
    Replaces all variables, numbers, commands inside formula by single chars, mainly question marks. 

    Parameters
    ----------
    formula : str
        The non-processed formula from the txt-file.

    Returns
    -------
    short_formula : str
        The shortened formula.
    var_list : list
        The list of extracted variables.

    """
    index_last = 0
    short_formula = ""
    var_list = []
    chars_in_formula = ["/", "*", "(", ")", "+", "-", "%"]
    tilde = ["sqrt"]
    #TODO abs -> $, (min, max) -> ° eigene Zeichen
    
    while True:
        index_next = search_char(formula, index_last, chars_in_formula)
        
        if index_next - index_last != 0:
            var_list.append(formula[index_last:index_next])
            if formula[index_last:index_next] in tilde:
                short_formula += "~"
            else:
                short_formula += "?"
                
        if index_next == len(formula):
            break
        
        short_formula += formula[index_next]
        index_last = index_next + 1
    
    return short_formula, var_list



def formula_to_tex(formula: str) -> str:
    """
    Mainly for calling a deeper level to convert the shortened formula to LaTeX and some post-processing.

    Parameters
    ----------
    formula : str
        The shortened formula, where variables are represented as question marks.

    Returns
    -------
    str
        The processed formula.

    """
    back = split_funct(formula)
    
    back = back.replace("*", mult_sign + " ")
    back = back.replace("(", "\\left(")
    back = back.replace(")", "\\right)")
    back = back.replace("%", "\bmod ")
    
    return back



def transform_vars(var_list: list, data: dict) -> (list, list, list):
    """
    Evaluates all variables in a formula and returns lists with their respective LaTeX-variables
    or values and units or only values.

    Parameters
    ----------
    var_list : list
        The list of variables in a formula.
    data : dict
        The main dictionary with all inputed variables.

    Returns
    -------
    var_list : list
        A list to input into the formula showing all LaTeX-variables.
    tex_val : list
        A list to input into the formula showing all values and units.
    py_val : list
        A list to into into the formula with all values.

    """
    #TODO abhängig von größe der Zahl Standardgenauigkeit ändern oder in Exponentialdarstellung wechseln
    tex_commands = ("e", "pi", "sin", "cos", "tan", "sqrt", "arccos", "arcsin", "arctan", "arcsinh", "arccosh", "arctanh", "sinh", "cosh", "tanh")
    trigo_list = ("sinD", "cosD", "tanD", "arccosD", "arcsinD", "arctanD", "sinG", "cosG", "tanG", "arccosG", "arcsinG", "arctanG")
    py_funct = ("min", "max", "abs")
    tex_val = var_list[:]
    py_val = var_list[:]
    
    #TODO use enumarate, would replace right site of equal-sign
    for i in range(len(var_list)):
        if is_float(var_list[i]):
            continue
        elif var_list[i] in tex_commands:
            py_val[i] = "np." + var_list[i]
            if var_list[i] != "e":
                tex_val[i] = var_list[i] = "\\" + var_list[i]
        elif var_list[i] in trigo_list:
            py_val[i] = "trigo." + var_list[i]
            tex_val[i] = var_list[i] = "\\" + var_list[:-1]
        elif var_list[i] in py_funct:
            if var_list[i] == "abs":
                tex_val[i] = var_list[i] = ""
            else:
                tex_val[i] = var_list[i] = "\\" + var_list[i]
        elif var_list[i] in data:
            py_val[i] = data[var_list[i]]["res"]
            #TODO genauigkeit aus data-dict oder standard aus settings, siehe oben
            tex_val[i] = "".join(("\\SI{", str(data[var_list[i]]["res"]), "}{", data[var_list[i]]["tex_un"].replace("-", ""), "}"))
            var_list[i] = data[var_list[i]]["tex_var"]
        #elif:
            #TODO prüfen ob item in bibs-dict
            #pass
        else:
            #TODO raise Exception
            pass
        
    return var_list, tex_val, py_val



def vars_in_short(short_formula: str, var_list: list) -> str:
    """Inputs all variables from var_list into short_formula"""
    for var in var_list:
        short_formula = short_formula.replace("?", str(var), 1)
        
    return short_formula




def py_eval(formula: str) -> float:
    """Calculates the result of the formula."""
    return eval(formula)
    


def main(formula: str, data: dict) -> (float, str, str):
    """
    Calculates the result of a formula and its LaTeX-representations given all other variables.

    Parameters
    ----------
    formula : str
        The formula, which should be processed.
    data : dict
        A dictionary containing all other variables and their results.

    Returns
    -------
    py_res : float
        The result of the formula.
    tex_var : str
        The formula, where the variables are replaced by their LaTeX-representations.
    tex_val : str
        The formula, where the variables are replaced by their values and units.

    """
    short_formula, var_list = find_vars(formula)
    
    short_tex = formula_to_tex(short_formula)
    
    tex_vars, tex_vals, py_vals = transform_vars(var_list, data)
    
    py_res = py_eval(vars_in_short(short_formula, py_vals))
    tex_var, tex_val = vars_in_short(short_tex, tex_vars), vars_in_short(short_tex, tex_vals)
    
    return (py_res, tex_var, tex_val)
    



# -*- coding: utf-8 -*-
"""
Created on Fri Mar  6 10:47:02 2020

@author: stefa
"""

"""
Befehle die die Trigonometrischen Funktionen auf Grad und Gon erweitern
"""
from numpy import pi, sin, cos, tan, arcsin, arccos, arctan

sinD = lambda arg: sin(arg*pi/180)
cosD = lambda arg: cos(arg*pi/180)
tanD = lambda arg: tan(arg*pi/180)

sinG = lambda arg: sin(arg*pi/200)
cosG = lambda arg: cos(arg*pi/200)
tanG = lambda arg: tan(arg*pi/200)

arcsinD = lambda arg: arcsin(arg)*180/pi
arccosD = lambda arg: arccos(arg)*180/pi
arctanD = lambda arg: arctan(arg)*180/pi

arcsinG = lambda arg: arcsin(arg)*200/pi
arccosG = lambda arg: arccos(arg)*200/pi
arctanG = lambda arg: arctan(arg)*200/pi



import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

VERSION = "0.0.1"
PACKAGE_NAME = "calc2tex"
AUTHOR = "Stefan Becherer"
AUTHOR_EMAIL = "becherer.stefan1@gmail.com"
URL = "https://github.com/stbech/calc2tex"

LICENSE = "MIT License"
DESCRIPTION = "Calculating formulas and displaying them in LaTeX"
LONG_DESCRIPTION = (HERE / "README.md").read_text()
LONG_DESC_TYPE = "text/markdown"

INSTALL_REQUIRES = [
    "numpy",
    "pint"
]

setup(name=PACKAGE_NAME,
      version=VERSION,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      long_description_content_type=LONG_DESC_TYPE,
      package_data={
          "": ["*.txt"]
          },
      author=AUTHOR,
      license=LICENSE,
      author_email=AUTHOR_EMAIL,
      url=URL,
      install_requires=INSTALL_REQUIRES,
      packages=find_packages()
      )


# -*- coding: utf-8 -*-
"""
Created on Sat Apr  4 08:34:03 2020

@author: stefa
"""
#TODO val_in erst bestimmen, wenn präzision bekannt oder Standardpräzision? -> Präzision in input.txt bestimmen
#TODO Exceptions-liste

from calc2tex import _back
from .settings import language, accuracy
import json


class Calc2tex:
    def __init__(self, filename: str, lang: str="DE"):
        self.data = _back.main(filename)
        self.lang = lang
    
    
    def to_json(self, output: str) -> None:
        """Exports the data-dictionary ta a json-file."""
        if "." not in output:
            output += ".json"
        with open(output, "w") as file:
            file.write(json.dumps(self.data, indent=4))
    
    
    def _search(self, py_var: str, lookup: str) -> str:
        """Searches for a variable and its sub-key inside the data-dictionary."""
        if py_var in self.data:
            return self.data[py_var][lookup]
        else:
            return "??"
            #TODO zum exceptions-dict hinzufügen
    
    
    def name(self, py_var: str) -> str:
        """Returns the name of a variable"""
        return self._search(py_var, "tex_var")
    
    
    def var(self, py_var: str) -> str:
        """Returns the formula with inputed variables."""
        return self._search(py_var, "var_in")
    
    
    def val(self, py_var: str) -> str:
        """Returns the formula with inputed values."""
        return self._search(py_var, "val_in")
    
    
    def raw(self, py_var: str, precision: int=3):
        """Returns the result with a certain precision"""
        return str(round(self._search(py_var, "res"), precision))
    
    
    def res(self, py_var: str, precision: int=accuracy) -> str:
        """Returns the result and its unit with a certain precision"""
        return "".join(("\\SI{", self.raw(py_var, precision), "}{", self.unit(py_var), "}"))
    
    
    def unit(self, py_var: str) -> str:
        """Returns the unit."""
        return self._search(py_var, "tex_un")
    
    
    def short(self, py_var: str, precision: int=accuracy) -> str:
        """Displays the name and value of a variable."""
        return " ".join((self.name(py_var), "=", self.res(py_var, precision)))
    
    
    def long(self, py_var: str, precision: int=accuracy) -> str:
        """Displays complete formula."""
        if self._search(py_var, "var") == "form":
            if self.var(py_var) == self.val(py_var):
                return " ".join((self.name(py_var), "=&", self.val(py_var), "=", self.res(py_var, precision)))
            else:
                return " ".join((self.name(py_var), "=&", self.var(py_var), "=", self.val(py_var), "=", self.res(py_var, precision)))
        else:
            return self.short(py_var, precision)
    
    
    def mult(self, first: str, last: str, precision: int=accuracy) -> str:
        """Displays multiple formulas at once."""
        #TODO var: val zulassen??, dafür table nur bis erstem var: form
        back = ""
        found = False
        
        for key, value in self.data.items():
            if key == first:
                found = True
            if found == True and self._search(key, "var") == "form":
                back += self.com(key, precision) + "\n"
            if key == last:
                break
                
        return back[:-2]
    
    
    #TODO zweispaltige Tabelle, dafür val-counter in read_file einbauen
    def table(self, columns: int=1) -> str:
        """
        Extracts the variables with predefined values from the main dictionary and parses them into a long string,
        which evaluates ta a table in LaTeX.

        Returns
        -------
        str
            The complete code block for displaying all input values.

        """
        start = ("\\begin{table}[htbp]", "    \\centering", "    \caption{"+language[self.lang]['table']['header']+"}", 
                 "    \\label{tab:input_val}", "    \\begin{tabular}{lcc}", " "*8 + "\\toprule", 
                 " "*8 + language[self.lang]['table']['var'] + " & " + language[self.lang]['table']['val'] + " & " + language[self.lang]['table']['unit'] + "\\\\", 
                 " "*8 + "\\midrule", "")
        end = (" "*8 + "\\bottomrule", "    \\end{tabular}", "\\end{table}")
        
        tab = "\n".join(start)
        for value in self.data.values():
            if type(value) == dict:
                if value["var"] == "val":
                    tab += " "*8 + "$" + value["tex_var"] + "$ & "
                    tab += str(value["res"]) + " & $\\si{"
                    tab += value["tex_un"] + "}$\\\\ \n"
            
        tab = "\n".join(tab, end)
        return tab
    